from typing import Optional

from pydantic import BaseModel, ValidationError

from gdshoplib.services.notion.models.props import PropModel


class OnRequestTable:
    def __init__(self, blocks, /, notion, parent):
        self.notion = notion
        self.parent = parent

        self._blocks = blocks
        self._rows = []

        self.__iter = None

    def _parse_rows(self):
        result = []
        for i, block in enumerate(self._blocks):
            row = []
            for r in block["table_row"]["cells"]:
                if not r:
                    row.append(None)
                    continue
                row.append(PropModel(page={}).get_type_data(r[0]))

            if row[4] == "Ошибка":
                continue

            row_result = dict(
                block_id=block["id"],
                name=row[0],
                link=row[1],
                price=row[2],
                quantity=row[3],
            )
            try:
                result.append(OnRequestTableRow(**row_result))
            except ValidationError as e:
                row_result["error"] = str(e)
                result.append(OnRequestTableInvalidRow(**row_result))
        return result

    @property
    def rows(self):
        if not self._rows:
            self._rows = self._parse_rows()
        return self._rows

    def __iter__(self):
        self.__iter = iter(self.rows)
        return self

    def __next__(self):
        return next(self.__iter)


class OnRequestTableRow(BaseModel):
    block_id: str
    name: str
    link: str
    price: float
    quantity: int
    error: Optional[str]


class OnRequestTableInvalidRow(BaseModel):
    block_id: str
    name: Optional[str]
    link: Optional[str]
    price: Optional[float]
    quantity: Optional[int]
    error: str



from gdshoplib.apps.crm.on_request import OnRequestTable
from gdshoplib.core.settings import CRMSettings
from gdshoplib.services.notion.database import Database
from gdshoplib.services.notion.page import Page


class Order(Page):
    SETTINGS = CRMSettings()

    def __init__(self, *args, **kwargs):
        self._on_request = None

        super(Order, self).__init__(*args, **kwargs)

    @classmethod
    def get(cls, sku):
        page = [
            page
            for page in Database(cls.SETTINGS.CRM_DB).pages(
                params={
                    "filter": {
                        "property": "ID заказа",
                        "rich_text": {
                            "equals": sku,
                        },
                    }
                }
            )
        ]
        if not page:
            return
        if len(page) > 1:
            raise OrderIDDuplicate

        page = page[0]

        return cls(page["id"], notion=page.notion, parent=page.parent)

    @classmethod
    def query(cls, filter=None, params=None, notion=None):
        for page in Database(cls.SETTINGS.CRM_DB, notion=notion).pages(
            filter=filter, params=params
        ):
            yield cls(page["id"], notion=page.notion, parent=page.parent)

    @property
    def on_request(self):
        if not self._on_request:
            self._on_request = OnRequestTable(
                [block for block in self.blocks(filter={"type": "table_row"})],
                notion=self.notion,
                parent=self,
            )

        return self._on_request


class OrderIDDuplicate(Exception):
    ...
